# -*- coding: utf-8 -*-
# Copyright (c) 2016 Sqreen. All Rights Reserved.
# Please refer to our terms for more information: https://www.sqreen.io/terms.html
""" Pyramid specific WSGI HTTP Request / Response stuff
"""

from itertools import chain
from logging import getLogger
from traceback import format_stack

from .ip_utils import get_real_user_ip

try:
    # Python 2
    import urlparse
except ImportError:
    import urllib.parse as urlparse

from .base import BaseRequest


LOGGER = getLogger(__name__)


class PyramidRequest(BaseRequest):

    def __init__(self, request):
        super(PyramidRequest, self).__init__()
        self.request = request

        # Cache for params
        self._query_params = None
        self._query_params_values = None
        self._form_params = None

    @property
    def query_params(self):
        if self._query_params is None:
            # Convert pyramid MultiDict to a normal dict with values as list
            self._query_params = dict(self.request.GET.dict_of_lists())
        return self._query_params

    @property
    def query_params_values(self):
        """ Return only query values as a list
        """
        if self._query_params_values is None:
            self._query_params_values = list(chain.from_iterable(self.query_params.values()))
        return self._query_params_values

    @property
    def form_params(self):
        # Convert pyramid MultiDict to a normal dict with values as list
        if self._form_params is None:
            self._form_params = dict(self.request.POST.dict_of_lists())
        return self._form_params

    @property
    def cookies_params(self):
        return dict(self.request.cookies)

    @property
    def client_ip(self):
        return get_real_user_ip(self.request.remote_addr,
                                self.request.environ.get('HTTP_X_FORWARDED_FOR', ''))

    @property
    def hostname(self):
        return self.request.host

    @property
    def method(self):
        return self.request.method

    @property
    def referer(self):
        return self.request.environ.get('HTTP_REFERER')

    @property
    def client_user_agent(self):
        return self.request.user_agent

    @property
    def path(self):
        return self.request.path

    @property
    def scheme(self):
        return self.request.scheme

    @property
    def server_port(self):
        return self.request.environ.get('SERVER_PORT')

    @property
    def remote_port(self):
        return self.request.environ.get('REMOTE_PORT')

    @property
    def view_params(self):
        return self.request.matchdict

    def get_header(self, name):
        """ Get a specific header
        """
        return self.request.environ.get(name)
